@@ -4,7 +4,7 @@
     Original ZynAddSubFX author Nasca Octavian Paul
     Copyright (C) 2002-2009 Nasca Octavian Paul
     Copyright 2009-2011, Alan Calvert
-    Copyright 2018-2019, Will Godfrey
+    Copyright 2018-2021, Will Godfrey
 
     This file is part of yoshimi, which is free software: you can redistribute
     it and/or modify it under the terms of the GNU Library General Public
@@ -68,14 +68,14 @@ static unsigned char presets[NUM_PRESETS][PRESET_SIZE] = {
 // todo: EarlyReflections, Prdelay, Perbalance
 
 Reverb::Reverb(bool insertion_, float *efxoutl_, float *efxoutr_, SynthEngine *_synth) :
-    Effect(insertion_, efxoutl_, efxoutr_, NULL, 0),
+    Effect(insertion_, efxoutl_, efxoutr_, NULL, 0, _synth),
     // defaults
 //    Pvolume(48),
     Ptime(64),
     Pidelay(40),
     Pidelayfb(0),
-    Prdelay(0),
-    Perbalance(64),
+//    Prdelay(0), // **** RHL ****
+//    Perbalance(64), // **** RHL ****
     Plpf(127),
     Phpf(0),
     Plohidamp(80),
@@ -88,7 +88,8 @@ Reverb::Reverb(bool insertion_, float *efxoutl_, float *efxoutr_, SynthEngine *_
     idelay(NULL),
     lpf(NULL),
     hpf(NULL), // no filter
-    synth(_synth)
+    lpffr(0, synth->samplerate),
+    hpffr(0, synth->samplerate)
 {
     setvolume(48);
     inputbuf = (float*)fftwf_malloc(synth->bufferbytes);
@@ -212,7 +213,7 @@ void Reverb::out(float *smps_l, float *smps_r)
     int i;
     for (i = 0; i < synth->sent_buffersize; ++i)
     {
-        inputbuf[i] = (smps_l[i] + smps_r[i]) / 2.0f;
+        inputbuf[i] = float(1e-20) + ((smps_l[i] + smps_r[i]) / 2.0f); // includes anti-denormal
         // Initial delay r
         if (idelay)
         {
@@ -597,7 +598,7 @@ float Revlimit::getlimits(CommandBlock *getData)
 {
     int value = getData->data.value;
     int control = getData->data.control;
-    int request = getData->data.type & 3; // clear upper bits
+    int request = getData->data.type & TOPLEVEL::type::Default; // clear flags
     int npart = getData->data.part;
     int presetNum = getData->data.engine;
     int min = 0;
