@@ -2,6 +2,7 @@
     InterChange.h - General communications
 
     Copyright 2016-2020 Will Godfrey
+    Copyright 2021,  Will Godfrey, Rainer Hans Liffers
 
     This file is part of yoshimi, which is free software: you can redistribute
     it and/or modify it under the terms of the GNU Library General Public
@@ -22,8 +23,6 @@
 #ifndef INTERCH_H
 #define INTERCH_H
 
-#include <jack/ringbuffer.h>
-
 #include "globals.h"
 #include "Interface/Data2Text.h"
 #include "Interface/RingBuffer.h"
@@ -41,10 +40,20 @@ extern std::string singlePath;
 extern int startInstance;
 
 
+/* compile time function log2 as per
+   https://hbfs.wordpress.com/2016/03/22/log2-with-c-metaprogramming          */
+static constexpr size_t log2
+                        (const size_t n)
+ {
+  return ((n < 2) ? 0 : 1 + log2 (n / 2));
+ }
+
+
 class InterChange : private DataText
 {
 
     private:
+        static constexpr size_t commandBlockSize = sizeof (CommandBlock);
         SynthEngine *synth;
 
     public:
@@ -54,14 +63,16 @@ class InterChange : private DataText
 
         CommandBlock commandData;
 #ifndef YOSHIMI_LV2_PLUGIN
-        ringBuff *fromCLI;
+        RingBuffer <9, log2 (commandBlockSize)> fromCLI;
 #endif
-        ringBuff *decodeLoopback;
-        ringBuff *fromGUI;
-        ringBuff *toGUI;
-        ringBuff *fromMIDI;
-        ringBuff *returnsBuffer;
-        ringBuff *muteQueue;
+        RingBuffer <10, log2 (commandBlockSize)> decodeLoopback;
+#ifdef GUI_FLTK
+        RingBuffer <10, log2 (commandBlockSize)> fromGUI;
+        RingBuffer <11, log2 (commandBlockSize)> toGUI;
+#endif
+        RingBuffer <10, log2 (commandBlockSize)> fromMIDI;
+        RingBuffer <10, log2 (commandBlockSize)> returnsBuffer;
+        RingBuffer <4, log2 (commandBlockSize)> muteQueue;
 
         void generateSpecialInstrument(int npart, std::string name);
         void mediate(void);
@@ -85,6 +96,8 @@ class InterChange : private DataText
         static void *_sortResultsThread(void *arg);
         pthread_t  sortResultsThreadHandle;
         void muteQueueWrite(CommandBlock *getData);
+        bool findManual(string& found);
+        std::string manualSearch(std::string dir2search, std::string path2match);
         void indirectTransfers(CommandBlock *getData, bool noForward = false);
         int indirectVector(CommandBlock *getData, SynthEngine *synth, unsigned char &newMsg, bool &guiTo, std::string &text);
         int indirectMidi(CommandBlock *getData, SynthEngine *synth, unsigned char &newMsg, bool &guiTo, std::string &text);
@@ -95,8 +108,6 @@ class InterChange : private DataText
         int indirectPart(CommandBlock *getData, SynthEngine *synth, unsigned char &newMsg, bool &guiTo, std::string &text);
         std::string formatScales(std::string text);
 
-        unsigned int lockTime;
-
         unsigned int swapRoot1;
         unsigned int swapBank1;
         unsigned int swapInstrument1;
@@ -137,6 +148,10 @@ class InterChange : private DataText
 
     private:
         bool commandSendReal(CommandBlock *getData);
+
+        int searchInst;
+        int searchBank;
+        int searchRoot;
 };
 
 #endif
