@@ -1,7 +1,8 @@
 /*
     MidiDecode.cpp
 
-    Copyright 2017-2020 Will Godfrey
+    Copyright 2017-2020, Will Godfrey
+    Copyright 2021, Will Godfrey, Rainer Hans Liffers
 
     This file is part of yoshimi, which is free software: you can redistribute
     it and/or modify it under the terms of the GNU Library General Public
@@ -26,6 +27,7 @@
 #include <string>
 #include <unistd.h>
 
+//#include <sys/time.h>
 
 #include "Interface/MidiDecode.h"
 #include "Interface/InterChange.h"
@@ -183,9 +185,22 @@ void MidiDecode::setMidiController(unsigned char ch, int ctrl, int param, bool i
      * Some controller values are >= 640 so they will be ignored by
      * later calls, but are passed as 128+ for this call.
      */
+/*
+    struct timeval tv1, tv2;
+    gettimeofday(&tv1, NULL);
+*/
     if (synth->midilearn.runMidiLearn(param, ctrl & 0xff, ch, in_place))
         return; // blocking while learning
-
+/*
+    gettimeofday(&tv2, NULL);
+    if (tv1.tv_usec > tv2.tv_usec)
+    {
+        tv2.tv_sec--;
+        tv2.tv_usec += 1000000;
+    }
+    int actual = (tv2.tv_sec - tv1.tv_sec) *1000000 + (tv2.tv_usec - tv1.tv_usec);
+    std::cout << "Delay " << std::to_string(actual) << "uS" << std::endl;
+*/
     /*
     * This is done here instead of in 'setMidi' so MidiLearn
     * handles all 14 bit values the same.
@@ -524,7 +539,7 @@ bool MidiDecode::nrpnProcessHistory(unsigned char nLow, unsigned char dHigh, uns
         else
             return true; // not an error but undefined
     }
-    synth->interchange.fromMIDI->write(putData.bytes);
+    synth->interchange.fromMIDI.write(putData.bytes);
     return true;
 }
 
@@ -533,6 +548,7 @@ void MidiDecode::nrpnDirectPart(int dHigh, int par)
 {
     CommandBlock putData;
     memset(&putData, 0xff, sizeof(putData));
+    bool partSet = false;
 
     switch (dHigh)
     {
@@ -547,33 +563,56 @@ void MidiDecode::nrpnDirectPart(int dHigh, int par)
                 synth->getRuntime().dataL = 0x80;
                 synth->getRuntime().dataH = 0x80;
             }
+            partSet = true;
             break;
 
         case 1: // Program Change
             setMidiProgram(synth->getRuntime().vectordata.Part | 0x80, par);
+            partSet = true;
             break;
 
         case 2: // Set controller number
             synth->getRuntime().vectordata.Controller = par;
             synth->getRuntime().dataL = par;
+            partSet = true;
+            //std::cout << "cont no " << int(par) << std::endl;
             break;
 
         case 3: // Set controller value
             setMidiController(synth->getRuntime().vectordata.Part | 0x80, synth->getRuntime().vectordata.Controller, par, false);
+            partSet = true;
+            //std::cout << "cont val " << int(par) << std::endl;
             break;
 
         case 4: // Set part's channel number
             putData.data.value = par;
-            putData.data.control = 5;
-            putData.data.part = synth->getRuntime().vectordata.Part;
+            putData.data.control = PART::control::midiChannel;
             break;
 
         case 5: // Set part's audio destination
             if (par > 0 and par < 4)
+            {
+                putData.data.value = par;
+                putData.data.control = PART::control::audioDestination;
+                //putData.data.parameter = 192; // this doesn't seem necessary
+            }
+            break;
+
+        case 8:
+            putData.data.control = PART::control::partToSystemEffect1;
+            putData.data.value = par;
+            break;
+        case 9:
+            putData.data.control = PART::control::partToSystemEffect2;
+            putData.data.value = par;
+            break;
+        case 10:
+            putData.data.control = PART::control::partToSystemEffect3;
+            putData.data.value = par;
+            break;
+        case 11:
+            putData.data.control = PART::control::partToSystemEffect4;
             putData.data.value = par;
-            putData.data.control = 120;
-            putData.data.part = synth->getRuntime().vectordata.Part;
-            putData.data.parameter = 192;
             break;
 
         case 64: // key shift
@@ -583,8 +622,7 @@ void MidiDecode::nrpnDirectPart(int dHigh, int par)
             else if (par > MAX_KEY_SHIFT)
                 par = MAX_KEY_SHIFT;
             putData.data.value = par;
-            putData.data.control = 35;
-            putData.data.part = synth->getRuntime().vectordata.Part;
+            putData.data.control = PART::control::keyShift;
             break;
         default:
             return;
@@ -592,6 +630,9 @@ void MidiDecode::nrpnDirectPart(int dHigh, int par)
     }
     if (dHigh < 4)
         return;
+    if (!partSet)
+        putData.data.part = synth->getRuntime().vectordata.Part;
+
     //std::cout << "part " << int(putData.data.part) << "  Chan " << int(par) << std::endl;
     putData.data.type = TOPLEVEL::type::Write | TOPLEVEL::type::Integer;
     putData.data.source = TOPLEVEL::action::toAll;
