@@ -56,6 +56,7 @@ namespace LEVEL{
         LFO, // amp/freq/filt
         Filter, // params only (slightly confused with env)
         Envelope, // amp/freq/filt/ (Sub only) band
+        Test, // special ops for Yoshimi-testsuite
     };
 }
 
@@ -116,7 +117,8 @@ namespace LISTS {
     list,
     bank,
     config,
-    mlearn
+    mlearn,
+    test,
     };
 }
 
@@ -126,11 +128,12 @@ static std::string basics [] = {
     "STop",          "all sound off",
     "RESet [s]",     "return to start-up conditions, 'ALL' clear MIDI-learn (if 'y')",
     "EXit [s]",      "tidy up and close Yoshimi (if 'y'), 'FOrce' instant exit regardless",
-    "RUN <s>",       "execute named command script",
-    "  WAIT <n>",    "1 to 1000 mS delay, within script only",
+    "RUN <s>",       "execute named command script forcing top level start",
+    "RUNLocal <s>",  "execute named command script from current context level",
+    "  WAIT <n>",    "1mS to 30,000mS delay, within script only",
     "..",            "step back one level",
     "/",             "step back to top level",
-    "end"
+    "@end"
 };
 
 static std::string toplist [] = {
@@ -154,6 +157,7 @@ static std::string toplist [] = {
     "  MLearn ...",             "enter editor context level",
     "  Bank ...",               "enter context level",
     "  COnfig ...",             "enter context level",
+    "  TESt ...",               "launch test calculations (for developers)",
     "  YOshimi <n>",            "read current instance or change to n",
     "  MONo <s>",               "main output mono/stereo (ON = mono, {other})",
     "  SYStem effects [n]",     "enter effects context level",
@@ -173,8 +177,9 @@ static std::string toplist [] = {
     "  SHift <n>",              "master key shift semitones (0 no shift)",
     "  DEtune <n>",             "master fine detune",
     "  SOlo [s]",               "channel 'solo' switch type (ROw, COlumn, LOop, TWoway, CHannel {other} off)",
-    "  SOlo CC <n>",             "incoming 'solo' CC number (type must be set first)",
-    "end"
+    "  SOlo CC <n>",            "incoming 'solo' CC number (type must be set first)",
+    "  CLear <n>",              "restore instrument on part n to default settings",
+    "@end"
 };
 
 static std::string configlist [] = {
@@ -220,7 +225,7 @@ static std::string configlist [] = {
     "Nrpn [s]",            "incoming NRPN (ON, {other})",
     "Log [s]",             "incoming MIDI CCs (ON, {other})",
     "SHow [s]",            "GUI MIDI learn editor (ON, {other})",
-    "end"
+    "@end"
 };
 
 static std::string banklist [] = {
@@ -234,7 +239,7 @@ static std::string banklist [] = {
 //    "Swap <n1> [n2]",            "Swap current bank with bank n1, (opt. in root n2)",
     "INstrument Rename <n> <s>", "change the name of slot n in the current bank",
     "INstrument SAve <n>",       "save current part's instrument to bank slot n",
-    "end"
+    "@end"
 };
 
 static std::string partlist [] = {
@@ -257,16 +262,16 @@ static std::string partlist [] = {
     "  (effect) <s>",      "the effect type",
     "  PREset <n>",        "set numbered effect preset to n",
     "    -- ",             "effect dependent controls",
-    "PRogram <s>/[s]<n>",  "loads instrument ID - CLear sets default,",
-    "","Group n selects from group list",
+    "PRogram <s>/[s]<n>",  "loads instrument ID - Group n selects from group list",
     "LAtest",              "Show most recent bank load or save",
     "NAme <s>",            "sets the display name the part can be saved with",
     "TYPe <s>",            "sets the instrument type",
-    "COPyright <s>",       "sets the instrument coyright message",
+    "COPyright <s>",       "sets the instrument copyright message",
     "INFo <s>",            "fills the comments info entry",
     "Humanise Pitch [n]",  "adds a small random pitch change at note_on",
     "Humanise Velocity [n]",  "adds a small random velocity change at note_on",
-    "Channel <n>",         "MIDI channel (> 32 disables, > 16 note off only)",
+    "CLear",               "Sets current instrument level parameters to default.",
+    "CHannel <n>",         "MIDI channel (> 32 disables, > 16 note off only)",
     "AFtertouch Chan <s1> [s2]", "Off, Filter (Down) + Peak (Down) + Bend (Down) + Modulation + Volume",
     "AFtertouch Key <s1> [s2]",  "Off, Filter (Down) + Peak (Down) + Bend (Down) + Modulation",
     "Destination <s>",     "jack audio destination (Main, Part, Both)",
@@ -289,7 +294,7 @@ static std::string partlist [] = {
     "SUBsynth ...",        "enter SubSynth context",
     "PADsynth ...",        "enter PadSynth context",
     "MCOntrol ...",        "enter MIDI controllers context",
-    "end"
+    "@end"
 };
 
 static std::string mcontrollist [] = {
@@ -325,7 +330,7 @@ static std::string mcontrollist [] = {
     "E Cutoff <n>",             "emulate filter cutoff controller",
     "E Q <n>",                  "emulate filter Q controller",
     "E BAndwidth <n>",          "emulate bandwidth controller",
-    "end"
+    "@end"
 };
 
 static std::string commonlist [] = {
@@ -367,7 +372,7 @@ static std::string commonlist [] = {
     "*-voice",                  "not AddVoice",
     "&",                        "AddSynth & PadSynth only",
     "#",                        "SubSynth & PadSynth only",
-    "end"
+    "@end"
 };
 
 static std::string addsynthlist [] = {
@@ -396,7 +401,7 @@ static std::string addsynthlist [] = {
     "FILter ...",               "enter Filter insert context",
     "ENVelope ...",             "enter Envelope insert context",
     "REsonance ...",            "enter Resonance context",
-    "end"
+    "@end"
 };
 
 static std::string addvoicelist [] = {
@@ -437,7 +442,7 @@ static std::string addvoicelist [] = {
     "LFO ...",              "enter LFO insert context",
     "FILter ...",           "enter Filter insert context",
     "ENVelope ...",         "enter Envelope insert context",
-    "end"
+    "@end"
 };
 
 static std::string addmodlist [] = {
@@ -464,11 +469,11 @@ static std::string addmodlist [] = {
     "FIXed <s>",            "set modulator frequency to 440Hz (ON, {other})",
     "SHift <n>",            "oscillator relative phase",
     "WAveform ...",         "enter the oscillator waveform context",
-    "end"
+    "@end"
 };
 
 // need to find a way to avoid this kind of duplication
-static std::string addmodnameslist [] = {"Off", "Morph", "Ring", "Phase", "Frequency", "Pulsewidth", "end"};
+static std::string addmodnameslist [] = {"Off", "Morph", "Ring", "Phase", "Frequency", "Pulsewidth", "@end"};
 
 static std::string subsynthlist [] = {
     "<ON/OFF>",                 "enables/disables the part",
@@ -502,7 +507,7 @@ static std::string subsynthlist [] = {
     "BAnd Scale <n>",           "bandwidth slope v frequency",
     "FILter ...",               "enter Filter insert context",
     "ENVelope ...",             "enter Envelope insert context",
-    "end"
+    "@end"
 };
 
 static std::string padsynthlist [] = {
@@ -564,7 +569,7 @@ static std::string padsynthlist [] = {
     "LFO ...",                  "enter LFO insert context",
     "FILter ...",               "enter Filter insert context",
     "ENVelope ...",             "enter Envelope insert context",
-    "end"
+    "@end"
 };
 
 static std::string  resonancelist [] = {
@@ -578,7 +583,7 @@ static std::string  resonancelist [] = {
     "Smooth",              "reduce range and sharpness of peaks",
     "CLear",               "set all points to mid level",
     "POints [<n1> [n2]]",  "show all or set/read n1 to n2",
-    "end"
+    "@end"
 };
 
 static std::string waveformlist [] = {
@@ -590,7 +595,7 @@ static std::string waveformlist [] = {
     "GAUss","",
     "DIOde","",
     "ABSsine","",
-    "PULsesine","",
+    "PSIne","",
     "STRetchsine","",
     "CHIrp","",
     "ASIne","",
@@ -625,32 +630,40 @@ static std::string waveformlist [] = {
     "ADdaptive Level <n>",      "adaptive power",
     "ADdaptive Par <n>",        "adaptive parameter",
     "APply",                    "fix settings (only for PadSynth)",
-    "end"
+    "@end"
 };
 
 static std::string LFOlist [] = {
     " - catgory -","",
-    "AMplitude",          "amplitude type",
-    "FRequency",          "frequency type",
-    "FIlter",             "filter type",
+    "AMplitude",             "amplitude type",
+    "FRequency",             "frequency type",
+    "FIlter",                "filter type",
     " - control -","",
-    "Rate <n>",         "frequency",
-    "Intensity <n>",    "depth",
-    "Start <n>",        "start position in cycle",
-    "Delay <n>",        "time before effect",
-    "Expand <n>",       "rate / note pitch",
-    "Continuous <s>",   "(ON, {other})",
-    "AR <n>",           "amplitude randomness",
-    "FR <n>",           "frequency randomness",
-    "Type <s>",         "LFO oscillator shape",
+    "Rate [<n>][<n1> <n2>]", "frequency or BPM [1 16] to [16 1], [2 3], [3 2]",
+    "Intensity <n>",         "depth",
+    "Start <n>",             "start position in cycle",
+    "Delay <n>",             "time before effect",
+    "Expand <n>",            "rate / note pitch",
+    "Bpm <s>",               "sync frequency to MIDI clock (ON, {other})",
+    "Continuous <s>",        "(ON, {other})",
+    "AR <n>",                "amplitude randomness",
+    "RR <n>",                "frequency randomness",
+    "Type <s>",              "LFO oscillator shape",
     "","SIne, TRiangle, SQuare, RUp (ramp up), RDown (ramp down), E1down (exp. 1), E2down (exp. 1)",
     "","SH (sample/hold), RSU (rand square up), RSD (rand square down)",
-    "e.g. S FI T RU",     "set filter type ramp up",
-    "end"
+    "e.g. S FI T RU",        "set filter type ramp up",
+    "@end"
 };
 // TODO need to find a way to safely (and efficiently) combine these
 static std::string LFOtype [] = {
-    "SIne", "TRiangle", "SQuare", "RUp (ramp up)", "RDown (ramp down)", "E1down (exp. 1)", "E2down (exp. 2)", "SH (sample/hold)", "RSU (rand square up)", "RSD (rand square dn)", "end"
+    "SIne", "TRiangle", "SQuare", "RUp (ramp up)", "RDown (ramp down)", "E1down (exp. 1)", "E2down (exp. 2)", "SH (sample/hold)", "RSU (rand square up)", "RSD (rand square dn)", "@end"
+};
+
+static std::string LFObpm [] = {
+"1/16 BPM", // space for expansion
+"1/16 BPM", "1/15 BPM", "1/14 BPM", "1/13 BPM", "1/12 BPM", "1/11 BPM", "1/10 BPM", "1/9 BPM", "1/8 BPM", "1/7 BPM", "1/6 BPM", "1/5 BPM", "1/4 BPM", "1/3 BPM", "1/2 BPM", "2/3 BPM", "1/1 BPM", "3/2 BPM", "2/1 BPM", "3/1 BPM", "4/1 BPM", "5/1 BPM", "6/1 BPM", "7/1 BPM", "8/1 BPM", "9/1 BPM", "10/1 BPM", "11/1 BPM", "12/1 BPM", "13/1 BPM", "14/1 BPM", "15/1 BPM", "16/1 BPM",
+"16/1 BPM", // space for expansion
+"Unknown BPM",
 };
 
 static std::string filterlist [] = {
@@ -696,7 +709,7 @@ static std::string filterlist [] = {
     "  FFrequency <n>", "center frequency of formant",
     "  FQ <n>",         "bandwidth of formant",
     "  FGain <n>",      "amplitude of formant",
-    "end"
+    "@end"
 };
 
 static std::string envelopelist [] = {
@@ -730,7 +743,7 @@ static std::string envelopelist [] = {
     "Insert <n1> <n2> <n3>", "insert point at n1 with X increment n2, Y value n3",
     "Delete <n>",            "remove point n",
     "Change <n1> <n2> <n3>", "change point n1 to X increment n2, Y value n3",
-    "end"
+    "@end"
 };
 
 static std::string reverblist [] = {
@@ -745,7 +758,7 @@ static std::string reverblist [] = {
     "TYPe <s>",         "reverb type (Random, Freeverb, Bandwidth)",
     "ROOm <n>",         "room size",
     "BANdwidth <n>",    "actual bandwidth (only for bandwidth type)",
-    "end"
+    "@end"
 };
 
 static std::string echolist [] = {
@@ -756,7 +769,8 @@ static std::string echolist [] = {
     "CROssover <n>",    "left-right crossover",
     "FEEdback <n>",     "echo feedback",
     "DAMp <n>",         "feedback damping",
-    "end"
+    "BPM <s>",          "delay BPM sync (ON {other})",
+    "@end"
 };
 
 static std::string choruslist [] = {
@@ -771,7 +785,9 @@ static std::string choruslist [] = {
     "FEEdback <n>",     "chorus feedback",
     "CROssover <n>",    "L/R routing",
     "SUBtract <s>",     "invert output (ON {other})",
-    "end"
+    "BPM <s>",          "LFO BPM sync (ON {other})",
+    "STArt <n>",        "LFO BPM phase start",
+    "@end"
 };
 
 static std::string phaserlist [] = {
@@ -790,7 +806,9 @@ static std::string phaserlist [] = {
     "HYPer <s>",        "hyper ?  (ON {other})",
     "OVErdrive <n>",    "distortion",
     "ANAlog <s>",       "analog emulation (ON {other})",
-    "end"
+    "BPM <s>",          "LFO BPM sync (ON {other})",
+    "STArt <n>",        "LFO BPM phase start",
+    "@end"
 };
 
 static std::string alienwahlist [] = {
@@ -805,7 +823,9 @@ static std::string alienwahlist [] = {
     "DELay <n>",        "LFO delay",
     "CROssover <n>",    "L/R routing",
     "RELative <n>",     "relative phase",
-    "end"
+    "BPM <s>",          "LFO BPM sync (ON {other})",
+    "STArt <n>",        "LFO BPM phase start",
+    "@end"
 };
 
 static std::string distortionlist [] = {
@@ -821,7 +841,7 @@ static std::string distortionlist [] = {
     "HIGh <n>",         "high pass filter",
     "STEreo <s>",       "stereo (ON {other})",
     "FILter <s>",       "filter before distortion",
-    "end"
+    "@end"
 };
 
 static std::string eqlist [] = {
@@ -833,7 +853,7 @@ static std::string eqlist [] = {
     "GAIn <n>",         "makeup gain",
     "Q <n>",            "filter Q",
     "STAges <n>",       "filter stages",
-    "end"
+    "@end"
 };
 
 static std::string dynfilterlist [] = {
@@ -848,10 +868,12 @@ static std::string dynfilterlist [] = {
     "INVert <s>",       "reverse effect of sensitivity (ON {other})",
     "RATe <n>",         "speed of filter change with amplitude",
     "FILter ...",       "enter dynamic filter context",
-    "end"
+    "BPM <s>",          "LFO BPM sync (ON {other})",
+    "STArt <n>",        "LFO BPM phase start",
+    "@end"
 };
 
-static std::string filtershapes [] = {"OFF" ,"ATA", "ASY", "POW", "SIN", "QNT", "ZIG", "LMT", "ULM", "LLM", "ILM", "CLI", "CLI", "AS2", "PO2", "SGM", "end"};
+static std::string filtershapes [] = {"OFF" ,"ATA", "ASY", "POW", "SIN", "QNT", "ZIG", "LMT", "ULM", "LLM", "ILM", "CLI", "CLI", "AS2", "PO2", "SGM", "@end"};
 
 static std::string learnlist [] = {
     "<n>",          "set current line number",
@@ -863,7 +885,7 @@ static std::string learnlist [] = {
     "MAx <n>",      "set maximum percentage",
     "LImit <s>",    "limit instead of compress (ON, {other})",
     "BLock <s>",    "inhibit others on this CC/Chan pair (ON, {other})",
-    "end"
+    "@end"
 };
 
 static std::string vectlist [] = {
@@ -875,7 +897,7 @@ static std::string vectlist [] = {
     "[X/Y] Control <n1> <n2>",  "sets n2 CC to use for X or Y feature n1 (2-4)",
     "OFF",                      "disable vector for this channel",
     "Name <s>",                 "text name for this complete vector",
-    "end"
+    "@end"
 };
 
 static std::string scalelist [] = {
@@ -896,7 +918,7 @@ static std::string scalelist [] = {
     "NAme <s>",           "internal name for this scale",
     "DEscription <s>",    "description of this scale",
     "CLEar",              "clear all settings and revert to standard scale",
-    "end"
+    "@end"
 };
 
 static std::string noteslist [] = { // from 21
@@ -917,7 +939,7 @@ static std::string loadlist [] = {
     "Patchset <s>",     "complete set of instruments from named file",
     "MLearn <s>",       "midi learned list from named file",
     "STate <s>",        "all system settings and patch sets from named file",
-    "end"
+    "@end"
 };
 
 static std::string savelist [] = {
@@ -929,7 +951,7 @@ static std::string savelist [] = {
     "MLearn <s>",       "midi learned list to named file",
     "STate <s>",        "all system settings and patch sets to named file",
     "Config",           "current configuration",
-    "end",
+    "@end",
 };
 
 static std::string listlist [] = {
@@ -946,7 +968,21 @@ static std::string listlist [] = {
     "History [s]",      "recent files (Patchsets, SCales, STates, Vectors, MLearn)",
     "Effects [s]",      "effect types ('all' include preset numbers and names)",
     "PREsets",          "all the presets for the currently selected effect",
-    "end"
+    "@end"
+};
+
+static std::string testlist [] = {
+    "NOte [n]",         "midi note to play for test",
+    "CHannel [n]",      "midi channel to use for the test note",
+    "VElocity [n]",     "velocity to use for note on/off",
+    "DUration [n]",     "overall duration for the test sound",
+    "HOldfraction [n]", "fraction of the duration to play sound before note off",
+    "REpetitions [n]",  "number of complete test cycles to play (minimum 1)",
+    "SCalestep [n]",    "semi-tones to move up/down when repeating a test note",
+    "BUffersize [n]",   "number of samples per Synth-call < global buffsize (=default)",
+    "TArget [s]",       "target file path to write sound data (empty: /dev/null)",
+    "EXEcute",          "actually trigger the test. Stops all other sound output.",
+    "@end"
 };
 
 static std::string replies [] = {
@@ -980,29 +1016,42 @@ static std::string fx_list [] = {
     "DIstortion",
     "EQ",
     "DYnfilter",
-    "end"
+    "@end"
 };
 
 static std::string type_list [] = {
     "undefined",
     "Piano",
+    "Bells and Chimes",
     "Chromatic Percussion",
     "Organ",
     "Guitar",
     "Bass",
     "Solo Strings",
     "Ensemble",
+    "Single Voice",
+    "Choir",
     "Brass",
     "Reed",
     "Pipe",
+    "Wind (other)",
     "Lead Synth",
-    "Pad Type Synth",
+    "Pad Synth",
+    "Warm Pad",
     "Synth Effects",
     "Ethnic",
     "Percussive",
     "Sound Effects",
-    "end"
+    "@end"
 };
+const int type_offset [] = {0, 1, -3, 2, 3, 4, 5, 6, 7, -6, -2, 8, 9, 10, -5, 11, 12, -4, 13, 14, 15, 16, 255};
+/*
+ * the number of the above entries must match
+ * @end and 255 are the recognised terminators
+ * only add negative numbers for backward compatibility
+ * they will resolve as 'undefined'
+ * note: can't use -1 as ID here
+ */
 
 static std::string fx_presets [] = {
     "1, off",
@@ -1017,30 +1066,29 @@ static std::string fx_presets [] = {
 };
 
 // effect controls
-static std::string effreverb [] = {"LEV", "PAN", "TIM", "DEL", "FEE", "none5", "none6", "LOW", "HIG", "DAM", "TYP", "ROO", "BAN", "end"};
-static std::string effecho [] = {"LEV", "PAN", "DEL", "LRD", "CRO", "FEE", "DAM",  "end"};
-static std::string effchorus [] = {"LEV", "PAN", "FRE", "RAN", "WAV", "SHI", "DEP", "DEL", "FEE", "CRO", "none11", "SUB", "end"};
-static std::string effphaser [] = {"LEV", "PAN", "FRE", "RAN", "WAV", "SHI", "DEP", "FEE", "STA", "CRO", "SUB", "REL", "HYP", "OVE", "ANA", "end"};
-static std::string effalienwah [] = {"LEV", "PAN", "FRE", "RAN", "WAV", "SHI", "DEP", "FEE", "DEL", "CRO", "REL", "end"};
-static std::string effdistortion [] = {"LEV", "PAN", "MIX", "DRI", "OUT", "WAV", "INV", "LOW", "HIG", "STE", "FIL", "end"};
-static std::string effdistypes [] = {"ATAn", "ASYm1", "POWer", "SINe", "QNTs", "ZIGzag", "LMT", "ULMt", "LLMt", "ILMt", "CLIp", "AS2", "PO2", "SGM", "end"};
+static std::string effreverb [] = {"LEV", "PAN", "TIM", "DEL", "FEE", "none5", "none6", "LOW", "HIG", "DAM", "TYP", "ROO", "BAN", "@end"};
+static std::string effecho [] = {"LEV", "PAN", "DEL", "LRD", "CRO", "FEE", "DAM", "none7", "none8", "none9", "none10", "none11", "none12", "none13", "none14", "none15", "none16", "BPM", "@end"};
+static std::string effchorus [] = {"LEV", "PAN", "FRE", "RAN", "WAV", "SHI", "DEP", "DEL", "FEE", "CRO", "none10", "SUB", "none12", "none13", "none14", "none15", "none16", "BPM", "@end"};
+static std::string effphaser [] = {"LEV", "PAN", "FRE", "RAN", "WAV", "SHI", "DEP", "FEE", "STA", "CRO", "SUB", "REL", "HYP", "OVE", "ANA", "none15", "none16", "BPM", "@end"};
+static std::string effalienwah [] = {"LEV", "PAN", "FRE", "RAN", "WAV", "SHI", "DEP", "FEE", "DEL", "CRO", "REL", "none11", "none12", "none13", "none14", "none15", "none16", "BPM", "@end"};
+static std::string effdistortion [] = {"LEV", "PAN", "MIX", "DRI", "OUT", "WAV", "INV", "LOW", "HIG", "STE", "FIL", "@end"};
+static std::string effdistypes [] = {"ATAn", "ASYm1", "POWer", "SINe", "QNTs", "ZIGzag", "LMT", "ULMt", "LLMt", "ILMt", "CLIp", "AS2", "PO2", "SGM", "@end"};
 static std::string effeq [] = {"LEV", "BAN", "FIL", "FRE", "GAI", "Q", "STA"};
-static std::string eqtypes [] = {"OFF", "LP1", "HP1", "LP2", "HP2", "BP2", "NOT", "PEAk", "LOW shelf", "HIGh shelf", "end"};
-static std::string effdynamicfilter [] = {"LEV", "PAN", "FRE", "RAN", "WAV", "SHI", "DEP", "SEN", "INV", "RAT", "FIL", "end"};
+static std::string eqtypes [] = {"OFF", "LP1", "HP1", "LP2", "HP2", "BP2", "NOT", "PEAk", "LOW shelf", "HIGh shelf", "@end"};
+static std::string effdynamicfilter [] = {"LEV", "PAN", "FRE", "RAN", "WAV", "SHI", "DEP", "SEN", "INV", "RAT", "FIL", "none11", "none12", "none13", "none14", "none15", "none16", "BPM", "@end"};
 
 // common controls
-static std::string detuneType [] = {"DEFault", "L35", "L10", "E100", "E1200", "end"};
+static std::string detuneType [] = {"DEFault", "L35", "L10", "E100", "E1200", "@end"};
 
 // waveform controls
 static std::string waveshape [] = {"Sine", "Triangle", "Pulse", "Saw", "Power", "Gauss", "Diode", "AbsSine", "PulseSine", "StretchSine", "Chirp", "AbsStretchSine", "Chebyshev", "Square", "Spike", "Circle", "HyperSec"};
-static std::string wavebase [] = {"SIN", "TRI", "PUL", "SAW", "POW", "GAU", "DIO", "ABS", "PSI", "SSI", "CHI", "ASI", "CHE", "SQU", "SPI", "CIR", "HYP", "end"};
+static std::string wavebase [] = {"SIN", "TRI", "PUL", "SAW", "POW", "GAU", "DIO", "ABS", "PSI", "SSI", "CHI", "ASI", "CHE", "SQU", "SPI", "CIR", "HYP", "@end"};
 static std::string basetypes [] = {"c2", "g2", "c3", "g3", "c4", "g4", "c5", "g5", "g6"};
-static std::string filtertype [] = {"OFF", "LP1", "HPA", "HPB", "BP1", "BS1", "LP2", "HP2", "BP2", "BS2", "COS", "SIN", "LSH", "SGM", "end"};
+static std::string filtertype [] = {"OFF", "LP1", "HPA", "HPB", "BP1", "BS1", "LP2", "HP2", "BP2", "BS2", "COS", "SIN", "LSH", "SGM", "@end"};
 static std::string adaptive [] = {"OFF", "ON", "SQU", "2XS", "2XA", "3XS", "3XA", "4XS", "4XA"};
 
 // misc controls
 static std::string historyGroup [] = {"IN", "PA", "SC", "ST", "VE", "ML"};
-static std::string instrumentGroupType [] = {"UNdefined", "PIano", "CHromatic percussion", "ORgan", "GUitar", "BAss", "SIngle strings", "ENsemble", "BRass", "REed", "ACcustic pipe", "LEad synth", "PAd synth", "SYnth effects", "EThnic", "PErcussive", "SOund effects"};
 
 static std::string subMagType [] = {"Linear", "40dB", "60dB", "80dB", "100dB"};
 static std::string subPadPosition [] = {"harmonic", "ushift", "lshift", "upower", "lpower", "sine", "power", "shift"};
