@@ -5,7 +5,7 @@
     Copyright (C) 2002-2005 Nasca Octavian Paul
     Copyright 2009-2011, Alan Calvert
     Copyright 2013, Nikita Zlobin
-    Copyright 2014-2020, Will Godfrey & others
+    Copyright 2014-2021, Will Godfrey & others
 
     This file is part of yoshimi, which is free software: you can redistribute
     it and/or modify it under the terms of the GNU Library General Public
@@ -74,11 +74,11 @@ namespace { // constants used in the implementation
         "Copyright 2002-2009 Nasca Octavian Paul and others, "
         "Copyright 2009-2011 Alan Calvert, "
         "Copyright 2012-2013 Jeremy Jongepier and others, "
-        "Copyright 2014-2019 Will Godfrey and others";
+        "Copyright 2014-2021 Will Godfrey and others";
     const string argline = "Yoshimi " + (string) YOSHIMI_VERSION;
     const char* argp_program_version = argline.c_str();
 
-    string stateText = "load saved state, defaults to '$HOME/" + EXTEN::config + "/yoshimi/yoshimi.state'";
+    string stateText = "load saved state, defaults to '$HOME/" + EXTEN::config + "/yoshimi/yoshimi-0.state'";
 
     static struct argp_option cmd_options[] = {
         {"alsa-audio",        'A',  "<device>",   1,  "use alsa audio output", 0},
@@ -100,6 +100,7 @@ namespace { // constants used in the implementation
         {"samplerate",        'R',  "<rate>",     0,  "set alsa audio sample rate", 0},
         {"oscilsize",         'o',  "<size>",     0,  "set AddSynth oscilator size", 0},
         {"state",             'S',  "<file>",     1,  stateText.c_str(), 0},
+        {"null",               13,  NULL,         0,  "use Null-backend without audio/midi", 0},
         #if defined(JACK_SESSION)
             {"jack-session-uuid", 'U',  "<uuid>",     0,  "jack session uuid", 0},
             {"jack-session-file", 'u',  "<file>",     0,  "load named jack session file", 0},
@@ -137,13 +138,14 @@ Config::Config(SynthEngine *_synth, int argc, char **argv) :
     alsaAudioDevice("default"),
     alsaMidiDevice("default"),
     loadDefaultState(false),
-    sessionStage(Session::Normal),
+    sessionStage(_SYS_::type::Normal),
     Interpolation(0),
     checksynthengines(1),
     xmlType(0),
     instrumentFormat(1),
     EnableProgChange(1), // default will be inverted
     toConsole(0),
+    consoleTextSize(12),
     hideErrors(0),
     showTimes(0),
     logXMLheaders(0),
@@ -197,10 +199,6 @@ Config::Config(SynthEngine *_synth, int argc, char **argv) :
     bRuntimeSetupCompleted(false),
     exitType(EXIT_SUCCESS)
 {
-    if (synth->getIsLV2Plugin())
-    {
-        rtprio = 4; // To force internal threads below LV2 host
-    }
     //else
         //fesetround(FE_TOWARDZERO); // Special thanks to Lars Luthman for conquering
                                // the heffalump. We need lrintf() to round
@@ -222,9 +220,13 @@ Config::Config(SynthEngine *_synth, int argc, char **argv) :
 
 bool Config::Setup(int argc, char **argv)
 {
-    AntiDenormals(true);
-    if (!synth->getIsLV2Plugin())
+    static bool torun = true;
+    if (torun) // only the first synth can read args
+    {
+        torun = false;
         loadCmdArgs(argc, argv);
+    }
+
     if (!loadConfig())
     {
         string message = "Could not load config. Using default values.";
@@ -232,15 +234,6 @@ bool Config::Setup(int argc, char **argv)
         Log("\n\n" + message + "\n");
     }
 
-    /* NOTE: we must not do any further init involving the SynthEngine here,
-     * since this code is invoked from within the SynthEngine-ctor.
-     */
-    if (synth->getIsLV2Plugin())
-    {
-        //skip further setup, which is irrelevant for lv2 plugin instance.
-        return true;
-    }
-
     switch (audioEngine)
     {
         case alsa_audio:
@@ -284,9 +277,7 @@ bool Config::Setup(int argc, char **argv)
 
 
 Config::~Config()
-{
-    AntiDenormals(false);
-}
+{;}
 
 
 void Config::flushLog(void)
@@ -295,7 +286,7 @@ void Config::flushLog(void)
     {
         while (LogList.size())
         {
-            std::cerr << LogList.front() << std::endl;
+            std::cout << LogList.front() << std::endl;
             LogList.pop_front();
         }
     }
@@ -311,8 +302,6 @@ void Config::clearPresetsDirlist(void)
 
 bool Config::loadConfig(void)
 {
-    //std::cout << "here load config" << std::endl;
-    string cmd;
     string homedir = string(getenv("HOME"));
     if (homedir.empty() || !isDirectory(homedir))
     {
@@ -346,7 +335,8 @@ bool Config::loadConfig(void)
     int thisInstance = synth->getUniqueId();
     defaultSession = defaultStateName + "-" + asString(thisInstance) + EXTEN::state;
     yoshimi += ("-" + asString(thisInstance));
-    if (thisInstance == 0 && sessionStage != Session::RestoreConf)
+    //std::cout << "\nsession >" << defaultSession << "<\n" << std::endl;
+    if (thisInstance == 0 && sessionStage != _SYS_::type::RestoreConf)
     {
         TextMsgBuffer::instance().init(); // sneaked it in here so it's early
 
@@ -368,25 +358,17 @@ bool Config::loadConfig(void)
                 }
             }
         }
-        //cout << presetDir << endl;
         definedBankRoot = localDir + "/found/";
         if (!isDirectory(definedBankRoot))
         { // only ever want to do this once
             if (createDir(definedBankRoot))
-            {
                 Log("Failed to create root directory '" + definedBankRoot + "'");
-            }
-            else
-            {
-                ;
-            }
-        //cout << definedBankRoot << endl;
         }
     }
 
     ConfigFile = ConfigDir + yoshimi;
 
-    if (thisInstance == 0 && sessionStage != Session::RestoreConf)
+    if (thisInstance == 0 && sessionStage != _SYS_::type::RestoreConf)
     {
         string newInstance0 = ConfigDir + yoshimi + EXTEN::instance;
         if (isRegularFile(baseConfig) && !isRegularFile(newInstance0), 0)
@@ -448,9 +430,11 @@ bool Config::loadConfig(void)
                     delete xml;
                 }
             }
-            if (thisInstance == 0 && sessionStage != Session::RestoreConf)
+            if (thisInstance == 0 && sessionStage != _SYS_::type::RestoreConf)
             {
-                if (lastXMLmajor < MIN_CONFIG_MAJOR || lastXMLminor < MIN_CONFIG_MINOR)
+                int currentVersion = lastXMLmajor * 10 + lastXMLminor;
+                int storedVersion = MIN_CONFIG_MAJOR * 10 + MIN_CONFIG_MINOR;
+                if (currentVersion < storedVersion)
                     oldConfig = true;
                 else
                     oldConfig = false;
@@ -460,10 +444,10 @@ bool Config::loadConfig(void)
 
     //std::cout << "Session Stage " << sessionStage << std::endl;
 
-    if (sessionStage == Session::RestoreConf)
+    if (sessionStage == _SYS_::type::RestoreConf)
         return true;
 
-    if (sessionStage != Session::Normal)
+    if (sessionStage != _SYS_::type::Normal)
     {
         XMLwrapper *xml = new XMLwrapper(synth, true);
         if (!xml)
@@ -473,10 +457,10 @@ bool Config::loadConfig(void)
             isok = xml->loadXMLfile(StateFile);
             if (isok)
             {
-                if (sessionStage == Session::StartupFirst)
-                    sessionStage = Session::StartupSecond;
-                else if (sessionStage == Session::JackFirst)
-                    sessionStage = Session::JackSecond;
+                if (sessionStage == _SYS_::type::StartupFirst)
+                    sessionStage = _SYS_::type::StartupSecond;
+                else if (sessionStage == _SYS_::type::JackFirst)
+                    sessionStage = _SYS_::type::JackSecond;
                 isok = extractConfigData(xml);
             }
             else
@@ -492,7 +476,7 @@ void Config::restoreConfig(SynthEngine *_synth)
     size_t tmpRoot = _synth->ReadBankRoot();
     size_t tmpBank = _synth->ReadBank();
     int tmpChanged = configChanged;
-    sessionStage = Session::RestoreConf;
+    sessionStage = _SYS_::type::RestoreConf;
 
     // restore old settings
     loadConfig();
@@ -519,11 +503,11 @@ void Config::defaultPresets(void)
         "/usr/share/zynaddsubfx/presets",
         "/usr/local/share/zynaddsubfx/presets",
         */
-        "end"
+        "@end"
     };
     int i = 0;
     int actual = 0;
-    while (presetdirs[i] != "end")
+    while (presetdirs[i] != "@end")
     {
         if (isDirectory(presetdirs[i]))
         {
@@ -538,7 +522,6 @@ void Config::defaultPresets(void)
 
 bool Config::extractBaseParameters(XMLwrapper *xml)
 {
-    //std::cout << "here load base" << std::endl;
     if (synth->getUniqueId() != 0)
         return true;
 
@@ -607,7 +590,6 @@ bool Config::extractBaseParameters(XMLwrapper *xml)
 
 bool Config::extractConfigData(XMLwrapper *xml)
 {
-    //std::cout << "Session Stage " << sessionStage << std::endl;
     if (!xml)
     {
         Log("extractConfigData on NULL");
@@ -623,21 +605,21 @@ bool Config::extractConfigData(XMLwrapper *xml)
      * default state must be first test as we need to abort
      * and fetch this instead
      */
-    if (sessionStage == Session::Normal)
+    if (sessionStage == _SYS_::type::Normal)
     {
         loadDefaultState = xml->getpar("defaultState", loadDefaultState, 0, 1);
         if (loadDefaultState)
         {
             xml->exitbranch(); // CONFIGURATION
             configChanged = true;
-            sessionStage = Session::Default;
+            sessionStage = _SYS_::type::Default;
             StateFile = defaultSession;
             Log("Loading default state");
             return true;
         }
     }
 
-    if (sessionStage != Session::InProgram)
+    if (sessionStage != _SYS_::type::InProgram)
     {
 
         if (!rateChanged)
@@ -648,6 +630,7 @@ bool Config::extractConfigData(XMLwrapper *xml)
             Oscilsize = xml->getpar("oscil_size", Oscilsize, MIN_OSCIL_SIZE, MAX_OSCIL_SIZE);
         single_row_panel = xml->getpar("single_row_panel", single_row_panel, 0, 1);
         toConsole = xml->getpar("reports_destination", toConsole, 0, 1);
+        consoleTextSize = xml->getpar("console_text_size", consoleTextSize, 11, 100);
         hideErrors = xml->getpar("hide_system_errors", hideErrors, 0, 1);
         showTimes = xml->getpar("report_load_times", showTimes, 0, 1);
         logXMLheaders = xml->getpar("report_XMLheaders", logXMLheaders, 0, 1);
@@ -705,9 +688,6 @@ bool Config::extractConfigData(XMLwrapper *xml)
         monitorCCin = xml->getparbool("monitor-incoming_CCs", monitorCCin);
         showLearnedCC = xml->getparbool("open_editor_on_learned_CC", showLearnedCC);
     }
-
-    //misc
-    //checksynthengines = xml->getpar("check_pad_synth", checksynthengines, 0, 1);
     if (tempRoot == 0)
         tempRoot = xml->getpar("root_current_ID", 0, 0, 127);
 
@@ -777,6 +757,7 @@ void Config::addConfigXML(XMLwrapper *xml)
 
     xml->addpar("single_row_panel", single_row_panel);
     xml->addpar("reports_destination", toConsole);
+    xml->addpar("console_text_size", consoleTextSize);
     xml->addpar("hide_system_errors", hideErrors);
     xml->addpar("report_load_times", showTimes);
     xml->addpar("report_XMLheaders", logXMLheaders);
@@ -1275,9 +1256,15 @@ int Config::SSEcapability(void)
     #endif
 }
 
+/*
+ * The code below has been replaced with specific anti-denormal code where needed.
+ * Although the new code is slightly less efficient it is compatible across platforms,
+ * where as the 'daz' processor code is not available on platforms such as ARM.
+ */
 
-void Config::AntiDenormals(bool set_daz_ftz)
+/*void Config::AntiDenormals(bool set_daz_ftz)
 {
+    return;
     if (synth->getIsLV2Plugin())
     {
         return;// no need to set floating point rules for lv2 - host should control it.
@@ -1300,7 +1287,7 @@ void Config::AntiDenormals(bool set_daz_ftz)
             _mm_setcsr(_mm_getcsr() & ~(0x0030|0x8000|0x0040|0x6000));
         }
     #endif
-}
+}*/
 
 
 /**
@@ -1451,17 +1438,25 @@ static error_t parse_cmds (int key, char *arg, struct argp_state *state)
         case 'S':
             if (arg)
             {
-                settings->sessionStage = Session::StartupFirst;
+                settings->sessionStage = _SYS_::type::StartupFirst;
                 settings->configChanged = true;
                 settings->StateFile = string(arg);
             }
             break;
 
+        case 13:
+            settings->configChanged = true;
+            settings->engineChanged = true;
+            settings->midiChanged = true;
+            settings->audioEngine = no_audio;
+            settings->midiEngine  = no_midi;
+            break;
+
 #if defined(JACK_SESSION)
         case 'u':
             if (arg)
             {
-                settings->sessionStage = Session::JackFirst;
+                settings->sessionStage = _SYS_::type::JackFirst;
                 settings->configChanged = true;
                 settings->StateFile = setExtension(string(arg), EXTEN::state);
             }
@@ -1508,49 +1503,19 @@ void GuiThreadMsg::processGuiMessages()
             if (!guiMaster)
                 std::cerr << "Error starting Main UI!" << std::endl;
             else
+            {
                 guiMaster->Init(guiMaster->getSynth()->getWindowTitle().c_str());
+
+                if (synth->getRuntime().audioEngine < 1)
+                    alert(synth, "Yoshimi could not connect to any sound system. Running with no Audio.");
+                if (synth->getRuntime().midiEngine < 1)
+                    alert(synth, "Yoshimi could not connect to any MIDI system. Running with no MIDI.");
+            }
         }
         else if (guiMaster)
         {
             switch(msg->type)
             {
-
-                case GuiThreadMsg::UpdateMaster:
-                    guiMaster->refresh_master_ui(msg->index);
-                    break;
-
-                case GuiThreadMsg::UpdateConfig:
-                    if (guiMaster->configui)
-                        guiMaster->configui->update_config(msg->index);
-                    break;
-
-                case GuiThreadMsg::UpdatePaths:
-                    guiMaster->updatepaths(msg->index);
-                    break;
-
-                case GuiThreadMsg::UpdatePart:
-                    guiMaster->updatepart();
-                    guiMaster->updatepanel();
-                    break;
-
-                case GuiThreadMsg::RefreshCurBank:
-                    if (msg->data && guiMaster->bankui)
-                    {
-                        if (msg->index == 1)
-                        {
-                            // special case for first synth startup
-                            guiMaster->bankui->readbankcfg();
-                            guiMaster->bankui->rescan_for_banks();
-                        }
-                        guiMaster->bankui->set_bank_slot();
-                        guiMaster->bankui->refreshmainwindow();
-                    }
-                    break;
-
-                case GuiThreadMsg::GuiCheck:
-                    guiMaster->checkBuffer();
-                    break;
-
                 default:
                     break;
             }
