@@ -472,12 +472,9 @@ int Microtonal::loadLine(const string& text, size_t &point, char *line, size_t m
     do {
         line[0] = 0;
         C_lineInText(text, point, line, maxlen);
-        if (line[0] == 0)
-        {
-            line[0] = '!'; // turn blank lines into comments
-            line[1] = 0; // we sould probably eliminate them when loading!
-        }
     } while (line[0] == '!');
+    if (line[0] < ' ')
+        return -5;
     return 0;
 }
 
@@ -532,6 +529,7 @@ int Microtonal::loadscl(const string& filename)
     octavesize = nnotes;
     std::swap(octave, tmpoctave);
     synth->setAllPartMaps();
+    synth->addHistory(filename, TOPLEVEL::XML::ScalaTune);
     return nnotes;
 }
 
@@ -656,6 +654,7 @@ int Microtonal::loadkbm(const string& filename)
     PrefNote = tmpNote;
     PrefFreq = tmpPrefFreq;
     synth->setAllPartMaps();
+    synth->addHistory(filename, TOPLEVEL::XML::ScalaMap);
     return tmpMapSize;
 }
 
