@@ -1,7 +1,7 @@
 /*
     MiscGui.cpp - common link between GUI and synth
 
-    Copyright 2016-2020 Will Godfrey & others
+    Copyright 2016-2021 Will Godfrey & others
 
     This file is part of yoshimi, which is free software: you can redistribute
     it and/or modify it under the terms of the GNU Library General Public
@@ -21,6 +21,7 @@
 
 #include "Misc/SynthEngine.h"
 #include "Misc/TextMsgBuffer.h"
+#include "Misc/NumericFuncs.h"
 #include "MiscGui.h"
 #include "MasterUI.h"
 
@@ -30,13 +31,13 @@
 #include <cairo.h>
 #include <cairo-xlib.h>
 
+using func::bpm2text;
 
 namespace { // Implementation details...
 
     TextMsgBuffer& textMsgBuffer = TextMsgBuffer::instance();
 }
 
-
 float collect_readData(SynthEngine *synth, float value, unsigned char control, unsigned char part, unsigned char kititem, unsigned char engine, unsigned char insert, unsigned char parameter, unsigned char offset, unsigned char miscmsg, unsigned char request)
 {
     unsigned char type = 0;
@@ -76,11 +77,10 @@ void collect_data(SynthEngine *synth, float value, unsigned char action, unsigne
     {
         if (collect_readData(synth, 0, PART::control::partBusy, part))
         {
-            fl_alert("Part %d is busy", int(part));
+            alert(synth, "Part " + to_string(part + 1) + " is busy");
             return;
         }
     }
-
     CommandBlock putData;
     putData.data.value = value;
     putData.data.control = control;
@@ -119,12 +119,8 @@ void collect_data(SynthEngine *synth, float value, unsigned char action, unsigne
                     }
                     else
                     {
-                        synth->getGuiMaster()->setmessage(UNUSED, false, "Can't learn this control");
+                        alert(synth, "Can't learn this control");
                         synth->getRuntime().Log("Can't MIDI-learn this control");
-                        /* can't use fl_alert here.
-                        * For some reason it goes into a loop on spin boxes
-                        * and runs menus up to their max value.
-                        */
                         type = TOPLEVEL::type::Learnable;
                     }
                 }
@@ -145,33 +141,68 @@ void collect_data(SynthEngine *synth, float value, unsigned char action, unsigne
 
     putData.data.type = type;
     putData.data.source = action;
-    //cout << "collect_data value " << value << "  action " << int(action)  << "  type " << int(type) << "  control " << int(control) << "  part " << int(part) << "  kit " << int(kititem) << "  engine " << int(engine) << "  insert " << int(insert)  << "  par " << int(parameter) << " par2 " << int(par2) << endl;
-    if (!synth->interchange.fromGUI->write(putData.bytes))
+    //cout << "collect_data value " << value << "  action " << int(action)  << "  type " << int(type) << "  control " << int(control) << "  part " << int(part) << "  kit " << int(kititem) << "  engine " << int(engine) << "  insert " << int(insert)  << "  par " << int(parameter) << " offset " << int(offset) << " msg " << int(miscmsg) << endl;
+    if (!synth->interchange.fromGUI.write(putData.bytes))
         synth->getRuntime().Log("Unable to write to fromGUI buffer.");
 }
 
+void alert(SynthEngine *synth, string message)
+{
+    synth->getGuiMaster()->query("", "", "", message);
+}
 
+int choice(SynthEngine *synth, string one, string two, string three, string message)
+{
+    return synth->getGuiMaster()->query(one, two, three, message);
+}
+
+string setfiler(SynthEngine *synth, string title, string name, bool save, int extension)
+{
+    return synth->getGuiMaster()->setfiler(title, name, save, extension);
+}
+
+string input_text(SynthEngine *synth, string label, string text)
+{
+    return synth->getGuiMaster()->setinput(label, text);
+}
+
+//static int keycheck = 0;
 void GuiUpdates::read_updates(SynthEngine *synth)
 {
     CommandBlock getData;
-    bool isChanged = false;
-    while (synth->interchange.toGUI->read(getData.bytes))
+    while (synth->interchange.toGUI.read(getData.bytes))
     {
         Fl::lock();
         decode_updates(synth, &getData);
         Fl::unlock();
-        isChanged = true;
     }
-    /*
-     * we perform all the updates that have occured in this refresh
-     * period then do just a single FLTK check for 'damage'
-     */
-    if (isChanged)
+    // and pull up to 5 entries from log
+    for (int i = 0; !synth->getRuntime().LogList.empty() && i < 5; ++i)
     {
-        Fl::lock();
-        Fl::check();
-        Fl::unlock();
+        synth->getGuiMaster()->Log(synth->getRuntime().LogList.front());
+        synth->getRuntime().LogList.pop_front();
     }
+    // direct key actions (not for LV2)
+    /*if (!synth->getIsLV2Plugin() && Fl::event_ctrl() && Fl::event_alt() && keycheck == 0)
+    {
+        if (Fl::event_key('H')) // immediate shutdown
+            synth->getGuiMaster()->masterwindow->do_callback();
+        else if (Fl::event_key('R')) // resize to fit
+        {
+            if(!synth->getIsLV2Plugin())
+            for (unsigned int id = 0; id < 31; ++id)
+            {
+                SynthEngine *_synth = synth->getSynthFromId(id);
+                if (_synth != synth)
+                    _synth->getGuiMaster()->rescaleMain();
+            }
+            synth->getGuiMaster()->rescaleMain();
+        }
+        keycheck = 90; // we need this to stop a stream of repeats
+
+    }
+    else if (keycheck > 0)
+        --keycheck;*/
 }
 
 
@@ -226,19 +257,13 @@ void GuiUpdates::decode_updates(SynthEngine *synth, CommandBlock *getData)
 
 //        cout << "Con " << int(control) << "  Kit " << int(kititem) << "  Eng " << int(engine) << "  Ins " << int(insert) << endl;
 
-    fl_line_style(FL_SOLID); // probably not needed
-    if (control == TOPLEVEL::control::textMessage) // just show a message
+    if (control == TOPLEVEL::control::textMessage) // just show a non-modal message
     {
         string name = textMsgBuffer.fetch(miscmsg);
         if (name.empty())
             synth->getGuiMaster()->message->hide();
         else
-        {
             synth->getGuiMaster()->setmessage(UNUSED, true, name, "Close");
-            //synth->getGuiMaster()->words->copy_label(name.c_str());
-            //synth->getGuiMaster()->cancel->hide();
-            //synth->getGuiMaster()->message->show();
-        }
         return;
     }
     if (npart == TOPLEVEL::section::scales)
@@ -322,12 +347,6 @@ void GuiUpdates::decode_updates(SynthEngine *synth, CommandBlock *getData)
     if (kititem >= NUM_KIT_ITEMS && kititem != UNUSED)
         return; // invalid kit number
 
-    if (kititem == UNUSED && engine == UNUSED && insert == UNUSED && control == PART::control::defaultInstrument) // special case for part clear
-    {
-        synth->getGuiMaster()->returns_update(getData);
-        return;
-    }
-
     if (insert != UNUSED || (control != PART::control::enable && control != PART::control::instrumentName))
     {
         if (synth->getGuiMaster()->partui->partname == DEFAULT_NAME)
@@ -567,6 +586,10 @@ void GuiUpdates::decode_updates(SynthEngine *synth, CommandBlock *getData)
     }
 }
 
+// static string freqBPMStr(float val)
+// This has been moved as bpm2text in Misc/NumericFuncs.h with other BPM shared functions
+
+
 string convert_value(ValueType type, float val)
 {
     float f;
@@ -611,6 +634,9 @@ string convert_value(ValueType type, float val)
             f = (powf(2.0f, val * 10.0f) - 1.0f) / 12.0f;
             return variable_prec_units(f, "Hz", 3);
 
+        case VC_LFOfreqBPM:
+            return bpm2text(val);
+
         case VC_LFOdepthFreq: // frequency LFO
             f=powf(2.0f,(int)val/127.0f*11.0f)-1.0f;
             return variable_prec_units(f, "cents", 2);
@@ -628,12 +654,13 @@ string convert_value(ValueType type, float val)
             f = ((int)val) / 127.0f * 4.0f + 0.005f;
             return(custom_value_units(f,"s",2));
 
-        case VC_LFOstartphase:
+        case VC_LFOstartphaseRand:
             if ((int)val == 0)
                 return("random");
-            else
-                return(custom_value_units(((int)val - 64.0f) / 127.0f
-                                      * 360.0f, "°"));
+            // fallthrough
+        case VC_LFOstartphase:
+            return(custom_value_units(((int)val - 64.0f) / 127.0f
+                                  * 360.0f, "°"));
         case VC_EnvelopeDT:
             // unfortunately converttofree() is not called in time for us to
             // be able to use env->getdt(), so we have to compute ourselves
@@ -767,6 +794,14 @@ string convert_value(ValueType type, float val)
             else
                 return(custom_value_units((val-96.0f)/96.0f*40.0f,"dB",1));
 
+        case VC_PartHumaniseDetune:
+            s = "Detune: ";
+            i = (int) val;
+            if (i == 0)
+                return s + "disabled";
+            else
+                return s + "between 0 and " + to_string(i) + " cents";
+
         case VC_PartHumaniseVelocity:
             s = "Attenuation: ";
             i = (int) val;
@@ -1016,6 +1051,9 @@ string convert_value(ValueType type, float val)
             f = (powf(2.0f, (int)val / 127.0f * 10.0f) - 1.0f) * 0.03f;
             return variable_prec_units(f, "Hz", 3);
 
+        case VC_FXlfofreqBPM:
+            return bpm2text(val / 127.0f);
+
         case VC_FXChorusDepth:
             f = powf(8.0f, ((int)val / 127.0f) * 2.0f) -1.0f; //ms
             return variable_prec_units(f, "ms", 2, true);
@@ -1350,6 +1388,11 @@ ValueType getLFOdepthType(int group)
     return(VC_plainValue);
 }
 
+ValueType getLFOFreqType(int bpmEnabled)
+{
+    return (bpmEnabled == 0) ? VC_LFOfreq : VC_LFOfreqBPM;
+}
+
 ValueType getFilterFreqType(int type)
 {
     switch(type)
@@ -1369,3 +1412,4 @@ ValueType getFilterFreqTrackType(int offset)
         default: return(VC_FilterFreqTrack1);
     }
 }
+
