@@ -2,7 +2,7 @@
     main.cpp
 
     Copyright 2009-2011, Alan Calvert
-    Copyright 2014-2020, Will Godfrey & others
+    Copyright 2014-2021, Will Godfrey & others
 
     This file is part of yoshimi, which is free software: you can
     redistribute it and/or modify it under the terms of the GNU General
@@ -19,20 +19,15 @@
 
 */
 
-#include <sys/mman.h>
 #include <iostream>
 #include <string>
 #include <stdio.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <termios.h>
-#include <time.h>
-
-#include <map>
-#include <list>
 #include <pthread.h>
-#include <thread>
-#include <semaphore.h>
+#include <atomic>
+
 #include <cstdio>
 #include <unistd.h>
 #include <readline/readline.h>
@@ -58,6 +53,9 @@
 extern map<SynthEngine *, MusicClient *> synthInstances;
 extern SynthEngine *firstSynth;
 
+// used by automated test launched via CLI
+std::atomic <bool> waitForTest{false};
+
 
 void mainRegisterAudioPort(SynthEngine *s, int portnum);
 int mainCreateNewInstance(unsigned int forceId);
@@ -139,7 +137,6 @@ static void *mainGuiThread(void *arg)
 #ifdef GUI_FLTK
 
     Fl::lock();
-
     const int textHeight = 15;
     const int textY = 10;
     const unsigned char lred = 0xd7;
@@ -209,9 +206,6 @@ static void *mainGuiThread(void *arg)
 #endif
     while (firstSynth == NULL); // just wait
 
-#ifdef GUI_FLTK
-    GuiThreadMsg::sendMessage(firstSynth, GuiThreadMsg::RefreshCurBank, 1);
-#endif
     if (firstRuntime->autoInstance)
         newBlock();
     while (firstRuntime->runSynth)
@@ -251,15 +245,17 @@ static void *mainGuiThread(void *arg)
 #ifdef GUI_FLTK
             if (bShowGui)
             {
-                for (int i = 0; !_synth->getRuntime().LogList.empty() && i < 5; ++i)
+                MasterUI *guiMaster = _synth->getGuiMaster(false);
+                if (guiMaster)
                 {
-                    MasterUI *guiMaster = _synth->getGuiMaster(false);
-                    if (guiMaster)
+                    if (guiMaster->masterwindow)
                     {
-                        guiMaster->Log(_synth->getRuntime().LogList.front());
-                        _synth->getRuntime().LogList.pop_front();
+                        guiMaster->checkBuffer();
+                        Fl::check();
                     }
                 }
+                else
+                    GuiThreadMsg::processGuiMessages();
             }
 #endif
         }
@@ -294,7 +290,6 @@ static void *mainGuiThread(void *arg)
                     }
                 }
                 Fl::wait(0.033333);
-                GuiThreadMsg::processGuiMessages();
             }
             else
 #endif
@@ -347,13 +342,7 @@ int mainCreateNewInstance(unsigned int forceId)
     {
         synth->setWindowTitle(musicClient->midiClientName());
         if (firstSynth != NULL) //FLTK is not ready yet - send this message later for first synth
-        {
             GuiThreadMsg::sendMessage(synth, GuiThreadMsg::NewSynthEngine, 0);
-        }
-        if (synth->getRuntime().audioEngine < 1)
-            fl_alert("Yoshimi can't find an available sound system. Running with no Audio");
-        if (synth->getRuntime().midiEngine < 1)
-            fl_alert("Yoshimi can't find an input system. Running with no MIDI");
     }
     else
         synth->getRuntime().toConsole = false;
@@ -446,7 +435,6 @@ int main(int argc, char *argv[])
         while (!Config.empty() && count < 16 && found < 3)
         { // no need to count beyond 16 lines!
             std::string line = func::nextLine(Config);
-            //std::cout << count << "  " << line  << std::endl;
             ++ count;
             if (line.find("enable_splash") != std::string::npos)
             {
@@ -462,6 +450,7 @@ int main(int argc, char *argv[])
             }
         }
     }
+
     if (isSingleMaster)
     {
         std::string firstText = runCommand("pgrep -o -x yoshimi", false);
@@ -490,8 +479,8 @@ int main(int argc, char *argv[])
     bool bExitSuccess = false;
     map<SynthEngine *, MusicClient *>::iterator it;
 
-    pthread_t thr;
-    pthread_attr_t attr;
+    pthread_t threadGui;
+    pthread_attr_t pthreadAttr;
     sem_t semGui;
 
     if (mainCreateNewInstance(0) == -1)
@@ -512,15 +501,15 @@ int main(int argc, char *argv[])
     }
     if (sem_init(&semGui, 0, 0) == 0)
     {
-        if (pthread_attr_init(&attr) == 0)
+        if (pthread_attr_init(&pthreadAttr) == 0)
         {
-            if (pthread_create(&thr, &attr, mainGuiThread, (void *)&semGui) == 0)
+            if (pthread_create(&threadGui, &pthreadAttr, mainGuiThread, (void *)&semGui) == 0)
             {
 #ifdef GUI_FLTK
                 guiStarted = true;
 #endif
             }
-            pthread_attr_destroy(&attr);
+            pthread_attr_destroy(&pthreadAttr);
         }
     }
 #ifdef GUI_FLTK
@@ -554,26 +543,37 @@ int main(int argc, char *argv[])
 #endif
 
     //create command line processing thread
-    pthread_t cmdThr;
+    pthread_t threadCmd;
 
     if (bShowCmdLine)
     {
-        if (pthread_attr_init(&attr) == 0)
+        if (pthread_attr_init(&pthreadAttr) == 0)
         {
-            if (pthread_create(&cmdThr, &attr, commandThread, (void *)firstSynth) == 0)
+            if (pthread_create(&threadCmd, &pthreadAttr, commandThread, (void *)firstSynth) == 0)
             {
                 ;
             }
-            pthread_attr_destroy(&attr);
+            pthread_attr_destroy(&pthreadAttr);
         }
     }
 
     void *ret;
-    pthread_join(thr, &ret);
+    pthread_join(threadGui, &ret);
     if (ret == (void *)1)
     {
         goto bail_out;
     }
+    if (waitForTest && threadCmd)
+    {   // CLI about to launch TestRunner for automated test
+        MusicClient* music = synthInstances[firstSynth];
+        if (music) music->Close(); // causes esp. JackClient to detach
+
+        pthread_join(threadCmd, &ret);
+        if (ret == (void *)1)
+        {
+            goto bail_out;
+        }
+    }
     bExitSuccess = true;
 
 bail_out:
